;=================================================
; Overworld Mario Tilemap Rewrite and Expand
; By Erik557
;
; Consists of two patches:
; - Overworld Mario Tilemap Rewrite: A complete
;   redo of the graphics routine used by Mario's
;   sprite in the overworld. Changes tables from
;   being 8x8 to 16x16, saving space in both OAM
;   proccessing, SNES proccessing and ROM space.
; - Overworld Mario Tilemap Expand: Allows for a
;   separate GFX file for the second player. Uh,
;   I guess that's a kinda misleading name lmao.
;=================================================

;===========
;  Defines
;===========

!EnableHalo   = 1           ;   1 to enable The Thing, 0 to disable
!HaloTile     = $38         ;   that's how the thing above mario's head when he dies is called, right
!EnableYoshi  = 1           ;   set to 0 if you don't use yoshi, to save space. Scroll below if you want to remap Yoshi's tilemap.
!FreeRAM      = $1486       ;   should be 4 consecutive bytes, holds what tile to upload per player.
!P1Palette    = $0A         ;\  should be from 8 to f.
!P2Palette    = $0B         ;/
!P1LETile     = $0A         ;\  tiles for the player on the lives exchanger screen.
!P2LETile     = $0C         ;/

;========================================================================================================
;   Scroll below to add more GFX files if you need to do so for some reason.
;   Note that you will need to modify some code by yourself if you want to, i.e., add 4 player support.
;   That being said, I don't recommend to touch anything below this point.
;========================================================================================================

math pri off                ;   probably not needed but

!RAM_P1TileLo = !FreeRAM           ;\
!RAM_P1TileHi = !FreeRAM+1         ; |
!RAM_P2TileLo = !FreeRAM+2         ; | for use below, no need to touch
!RAM_P2TileHi = !FreeRAM+3         ; |
!P1Pal        = !P1Palette-8*2     ; |
!P2Pal        = !P2Palette-8*2     ;/

;   Hijacks and cleanup

org $04862E          ;   free up 1862 bytes of freespace (almost half of it is tables. because player gfx is drawn as 8x8)
       pad $048D74

org $0485BB                        ;   Optimize code to not waste space:
       LDA #$08                    ;\  store x speed, 1 byte saved because it was STA $007B???
       STA $7B                     ;/
       PEA $0400                   ;\
       PLB                         ; | jump to whatever, 1 byte saved with PEA
       JSL $00CEB1                 ; |
       PLB                         ;/
       LDA #$03                    ;\  maximum priority
       STA $13F9                   ;/
       JSL $00E2BD                 ;   mario's gfx routine or something
       autoclean JML setTileUpload ;   select tile to upload

org $008222
       autoclean JML uploadTile    ;   actually upload the tile to vram

;   Bug fixes and misc.

org $0491E5                        ;   don't increase game mode yet
       RTS
org $04F57B                        ;   tiles for players in the lives exchanger
       db !P1LETile,$30+!P1Pal
org $04F581
       db !P2LETile,$30+!P2Pal

;   Main patch starts here.

freecode

P1GFX:
       incbin OWPlayer1GFX.bin     ;\  gfx files
P2GFX:                             ; |
       incbin OWPlayer2GFX.bin     ;/  should be in the same bank

;============================
;   Set what tile to upload
;============================

setTileUpload:
       LDA $0DB3            ;\
       EOR #$01             ; | get the player that's not currently playing * 2 in Y
       ASL                  ; |
       TAY                  ;/
       LDX $0DB2            ;\  don't get data for player 2 if not on a two player game
       BEQ .onePlayer       ;/
       TYA
       LSR
       TAY
       LDX #$00
       LDA $0DB4,y
       BMI .noAnimP2
       LDA $14              ;\
       LSR #3               ; | get a frame value
       AND #$01             ;/
       TAX
.noAnimP2
       TXA
       CLC                  ;\
       ADC #$02             ; | (+ 2) * 2
       ASL                  ;/
       TAX                  ;   get it in X
       TYA
       ASL
       TAY
       LDA.l UploadTile,x   ;\
       STA !RAM_P1TileLo,y  ; | get data of what tile to upload
       LDA.l UploadTile+1,x ; |
       STA !RAM_P1TileHi,y  ;/
.onePlayer
       TYA                  ;\
       EOR #$02             ; | now use data of the current character playing
       LSR                  ; |
       TAY                  ;/
       LDA $0DBA,y
       BEQ .normalTile
       TYA
       ASL
       TAY
       LDA $14
       LSR #3
       AND #$01
       CLC
       ADC $1F13,y
       ASL
       TAX
       LDA.l YoshiTile,x
       STA !RAM_P1TileLo,y
       LDA.l YoshiTile+1,x
       STA !RAM_P1TileHi,y
       LDA $1F13,y
       BRA .return

.normalTile
       TYA
       ASL
       TAY
.normalTile2
       LDA $1F13,y          ;\
       CMP #$16             ; |
       BNE +                ; | make value 0x16 use the same data as value 0x14
       EOR #$02             ; |
+      STA $00              ;/
.animate
       LDA $14              ;\
       LSR #3               ; | get a frame value
       AND #$01             ;/
       CLC                  ;\
       ADC $00              ; | add the current action, multiply by 2 and get it into X
       ASL                  ; |
       TAX                  ;/
       LDA.l UploadTile,x   ;\
       STA !RAM_P1TileLo,y  ; | get data of what tile to upload
       LDA.l UploadTile+1,x ; |
       STA !RAM_P1TileHi,y  ;/
       LDA $00              ;\
.return                     ; |
       CMP #$10             ; | check if the player is entering a new level
       BCC .normalRet       ; |
       CMP #$13             ; |
       BCS .normalRet       ;/
       INC $0100            ;\
       PLA                  ; | increase gamemode and change the return address if so
       PLA                  ; |
       PEA $840F            ;/
.normalRet
       LDA #$06             ;   restore
       JML $0485D5

UploadTile:                                                    ;   DON'T TOUCH THIS TABLE UNLESS YOU KNOW WHAT YOU'RE DOING.
       dw $0000,$0040,$0080,$00C0,$0100,$0140,$0100,$0140      ;   On land:  Going up, going down/still, going left, going right
       dw $0180,$01C0,$0400,$0440,$0480,$04C0,$0480,$04C0      ;   On water: Going up, going down/still, going left, going right
       dw $0500,$0500,$0540,$0540,$0580,$05C0                  ;   Enter level (land), enter level (water), climbing

YoshiTile:
       dw $0800,$0800,$0080,$0080,$0840,$0840,$0840,$0840
       dw $0800,$0800,$0080,$0080,$0840,$0840,$0840,$0840
       dw $0500,$0500,$0500,$0500,$0800,$0800,$0800,$0800

;===============================
;   Upload player tile to VRAM
;===============================

uploadTile:
;   Initial setup
       REP #$21                    ;   16-bit A, clear carry
       LDA !RAM_P1TileLo           ;   get VRAM address for tile to upload in A
       STA $00                     ;   store on scratch RAM       
       LDY #$80                    ;\  increment on 2119 write
       STY $2115                   ;/
       LDA #$1801                  ;\  write to VRAM, 2 regs write once
       STA $4300                   ;/
       LDY.b #uploadTile>>16       ;\ bank with the graphics
       STY $4304                   ;/
       LDY #$01                    ;  DMA in channel 0

;   Top tiles
       LDA #$6060                  ;\  upload to tile 0x306
       STA $2116                   ;/
       LDA.w #P1GFX                ;\
       ADC $00                     ; | get location of the file depending on player for top 8x8s
       STA $4302                   ;/
       LDA #$0040                  ;\  2 8x8 tiles
       STA $4305                   ;/
       STY $420B                   ;   start dma

;   Bottom tiles
       LDA #$6160                  ;\  upload to tile 0x316
       STA $2116                   ;/
       LDA.w #P1GFX                ;\
       CLC                         ; |
       ADC $00                     ; | get location of the file depending on player + 0x0200 for bottom 8x8s
       ADC #$0200                  ; |
       STA $4302                   ;/
       LDA #$0040                  ;\  2 8x8 tiles
       STA $4305                   ;/
       STY $420B                   ;   start dma

;   2 Player game check
       SEP #$20                    ;   8-bit A
       LDA $0DB2                   ;\  stop if not on a two player game
       BEQ .end                    ;/
       REP #$21                    ;   16-bit A, clear carry
       LDA !RAM_P2TileLo           ;   get VRAM address for tile to upload in A
       STA $00                     ;   store on scratch RAM       

;   Top tiles
       LDA #$6080                  ;\  upload to tile 0x308
       STA $2116                   ;/
       LDA.w #P2GFX                ;\
       ADC $00                     ; | get location of the file depending on player for top 8x8s
       STA $4302                   ;/
       LDA #$0040                  ;\  2 8x8 tiles
       STA $4305                   ;/
       STY $420B                   ;   start dma

;   Bottom tiles
       LDA #$6180                  ;\  upload to tile 0x318
       STA $2116                   ;/
       LDA.w #P2GFX                ;\
       CLC                         ; |
       ADC $00                     ; | get location of the file depending on player + 0x0200 for bottom 8x8s
       ADC #$0200                  ; |
       STA $4302                   ;/
       LDA #$0040                  ;\  2 8x8 tiles
       STA $4305                   ;/
       STY $420B                   ;   start dma

;   End
       SEP #$20                    ;   8-bit A
.end
       LDA $13D9                   ;\  restore
       CMP #$0A                    ;/
       JML $008227                 ;   return


;=========================
;   New graphics routine
;=========================

org $04862E
OWPlayerGFX:
       REP #$20
       LDA $1F17
       SEC
       SBC $1A
       CMP #$0100
       BCS .dontDrawP1
       STA $00              ;   $00 contains player 1 X position relative to the screen
       LDA $1F19
       SEC
       SBC $1C
       CMP #$0100
       BCS .dontDrawP1
       STA $01              ;   $01 contains player 1 Y position relative to the screen
       LDY #$08
       LDX #$00
       JSR DrawPlayer
       if !EnableHalo == 1
              LDY $0DB4
              BPL .noP1Halo
              LDX #$10
              PEA.w .dontDrawP1-1  ;   ignore the yoshi code
              JMP DrawHalo
       .noP1Halo
       endif
       if !EnableYoshi == 1
              LDY $0DBA
              BEQ .dontDrawP1
              TYA
              LSR
              STA $04
              LDA $1F13
              LSR
              TAY
              LDA OAMSlotYoshiP1,y
              TAX                  ;   will clear hi so it won't matter if theres garbage
              LDY #$00
              JSR DrawYoshi
       endif
.dontDrawP1
       LDY $0DB2
       BEQ .dontDrawP2
       LDA $1F1B
       SEC
       SBC $1A
       CMP #$0100
       BCS .dontDrawP2
       STA $00              ;   $00 contains player 2 X position relative to the screen
       LDA $1F1D
       SEC
       SBC $1C
       CMP #$0100
       BCS .dontDrawP2
       STA $01              ;   $01 contains player 2 Y position relative to the screen
       LDY #$0C
       LDX #$02
       JSR DrawPlayer
       if !EnableHalo == 1
              LDY $0DB5
              BPL .noP2Halo
              LDX #$14
              PEA.w .dontDrawP2-1  ;   ignore the yoshi code
              JMP DrawHalo
       .noP2Halo
       endif
       if !EnableYoshi == 1
              LDY $0DBB
              BEQ .dontDrawP2
              TYA
              LSR
              STA $04
              LDA $1F15
              LSR
              TAY
              LDA OAMSlotYoshiP2,y
              TAX                  ;   will clear hi so it won't matter if theres garbage
              LDY #$02
              JSR DrawYoshi
       endif
.dontDrawP2
       SEP #$20
       RTS

DrawPlayer:
       STY $1F14
       db $A9,!P2Pal,$08
       CPY #$0C
       BEQ +
       db $A9,!P1Pal,$06
+      STA $02
       SEP #$20
       LDA $1F13,x
       CMP #$16
       BNE +
       EOR #$02
+      LSR
       PHA
       TXA
       LSR
       TAX
       PLA
       LDY $0DBA,x
       BEQ +
       ASL
       PHA
       LDA $14
       LSR #3
       AND #$01
       CLC
       ADC $01,s
       PLX
       CLC
       ADC #$0B
+      TAY
       LDX $1F14
       LDA $00
       CLC
       ADC XDisp,y
       STA $029C,x
       LDA $01
       CLC
       ADC YDisp,y
       STA $029D,x
       LDA $03
       STA $029E,x
       LDA Props,y
       ORA $02
       STA $029F,x
       TXA
       LSR #2
       TAX
       LDA #$02
       STA $0447,x
       LDX $0DD6
       REP #$20
       RTS

XDisp:
       db $F8,$F8,$F8,$F8
       db $F8,$F8,$F8,$F8
       db $F8,$F8,$F8

       db $F8,$F8,$F8,$F8,$FF,$FF,$F1,$F1
       db $F8,$F8,$F8,$F8,$FF,$FF,$F1,$F1
       db $F8,$F8,$F8,$F8,$F8,$F8
YDisp:
       db $F2,$F2,$F2,$F2
       db $F7,$F7,$F7,$F7
       db $F2,$F7,$F2

       db $EC,$ED,$EB,$EA,$EE,$ED,$EE,$ED
       db $EF,$F0,$EF,$F0,$F3,$F2,$F3,$F2
       db $EA,$EA,$F0,$F0,$EC,$ED

Props:
       db $20,$20,$20,$60
       db $20,$20,$20,$60
       db $20,$20,$20

       db $20,$20,$20,$20,$20,$20,$60,$60
       db $20,$20,$20,$60,$20,$20,$60,$60
       db $20,$20,$20,$20,$20,$20

if !EnableHalo == 1
       DrawHalo:
              LDA $0294,x
              STA $02
              SEP #$20
              LDY #$01
       -      LDA $02
              CLC
              ADC HaloXDisp,y
              STA $029C,x
              LDA $03
              SEC
              SBC #$08
              STA $029D,x
              LDA.b #!HaloTile
              STA $029E,x
              LDA HaloProps,y
              STA $029F,x
              PHX
              TXA
              LSR #2
              TAX
              STZ $0447,x
              PLX
              INX #4
              DEY
              BPL -
              REP #$20
              RTS

       HaloXDisp:
              db $00,$08
       HaloProps:
              db $20,$60
endif

if !EnableYoshi == 1

       OAMSlotYoshiP1:
              db $18,$00,$18,$18
              db $18,$00,$18,$18
              db $00,$00,$18,$18
       OAMSlotYoshiP2:
              db $1C,$04,$1C,$1C
              db $1C,$04,$1C,$1C
              db $04,$04,$1C,$1C

       DrawYoshi:
              SEP #$20
              STZ $05
              PHY
              LDY $04
              LDA.w YoshiProps-2,y
              STA $04
              PLY
              LDA $1F13,y
              CMP #$04
              BEQ .right
              CMP #$0C
              BNE .noRight
       .right
              LDA #$40
              TSB $04
              LDA $1F13,y
       .noRight
              CMP #$10
              BEQ .noAnimYoshi
              CMP #$12
              BEQ .noAnimYoshi
              CMP #$08
              BEQ .return
       .animYoshi
              PHY
              LDY #$40
              LDA $14
              LSR #3
              AND #$01
              STA $05
              BNE +
              LDY #$00
       +      TYA
              ORA $04
              STA $04
              PLY
              LDA $1F13,y
       .noAnimYoshi
              PHA
              PHY
              CLC
              ADC $05
              TAY
              LDA YoshiTilemap,y
              STA $05
              PLY
              LDA $1F13,y
              CMP #$06
              BEQ +
              CMP #$0E
              BNE ++
       +      LDA #$40
              TRB $04
       ++     PLA
              LSR
              PHY
              TAY
              LDA $00
              CLC
              ADC #$F8
              STA $029C,x
              LDA $01
              CLC
              ADC YoshiYDisp,y
              STA $029D,x
              LDA $05
              LDY #$00
              CMP #$24
              STA $029E,x
              BEQ +
              LDY #$40
       +      TYA
              EOR $04
              STA $029F,x
              PHX
              TXA
              LSR #2
              TAX
              LDA #$02
              STA $0447,x
              PLX
              PLY
              LDA $1F13,y
              CMP #$0C
              BEQ +
              CMP #$0E
              BNE .return
       +      SEC
              SBC #$0C
              JSR WaterTileFix
       .return
              CMP #$08
              BNE +
              JSR Water2Fix
       +      REP #$20
              RTS

       YoshiProps:
              db $30,$32,$34,$36
       YoshiYDisp:
              db $F2,$F2,$F2,$F2
              db $F7,$F7,$F7,$F7
              db $F2,$F7,$F2,$F2
       YoshiTilemap:
              db $2E,$2E,$2E,$2E,$40,$42,$40,$42
              db $20,$24,$20,$24,$64,$66,$64,$66
              db $2E,$2E,$20,$20,$2E,$2E,$2E,$2E

       WaterTileFix:
              STA $06
              LDA $14
              LSR #3
              AND #$01
              CLC
              ADC $06
              TAY
              LDA $00
              CLC
              ADC WaterXDisp,y
              STA $0240,x
              LDA $01
              DEC
              STA $0241,x
              LDA WaterTile,y
              STA $0242,x
              LDA #$36
              STA $0243,x
              TXA
              LSR #2
              TAX
              STZ $0430,x
              RTS

       WaterXDisp:
              db $00,$00,$F8,$F8
       WaterTile:
              db $75,$77,$74,$76

       Water2Fix:
              LDA $00
              STA $0230,x
              CLC
              ADC #$F8
              STA $0238,x
              LDA $01
              DEC
              STA $0231,x
              STA $0239,x
              LDA $14
              LSR #3
              AND #$01
              ASL
              TAY
              LDA Water2,y
              STA $0232,x
              LDA.w Water2+1,y
              STA $023A,x
              LDA #$36
              STA $0233,x
              ORA #$70
              STA $023B,x
              TXA
              LSR #2
              TAX
              STZ $042C,x
              STZ $042E,x
              RTS

       Water2:
              db $75,$75,$76,$77
endif

warnpc $048D74
padbyte $EA : pad $048D74

